#include <stdafx.h>

#include "Image.h"


inline double getDecimal(double d)
{
	return d - (int)d;
}

// Resize the current image
void SimpleImage::resample(const SimpleImage& i)
{
	// Currently only implemented box filter

	// Determine multiple to convert coordinates
	// from current image to i.
	double wMult = (double)i.w() / w_;
	double hMult = (double)i.h() / h_;

	// double TOL = 0.0001;
	
	// Determine each image in the destination image by applying
	// box filter on source image i.
	for (int y = 0; y < h_; y++)
	{
		// Determine y bounds of image
		double yiT = y*hMult;
		double yiB = yiT+hMult;
		int yiTi = (int)yiT;
		int yiBi = (int)yiB;

		// Ensure within image extents
		if (yiBi+1 >= i.h())
		{
			yiBi = yiBi-1;
		}

		// Determine how much of first and last row to include
		double pixMult1y = 1.0-(yiT-yiTi);
		double pixMult2y = yiB-yiBi;

		// Apply box filter to the current row of pixels
		for (int x = 0; x < w_; x++)
		{
			// Determine x bounds of image
			double xiT = x*wMult;
			double xiB = xiT+wMult;
			int xiTi = (int)xiT;
			int xiBi = (int)xiB;
			
			// Determine how much of first and last pixel to include
			double pixMult1x = 1.0-(xiT-xiTi);
			double pixMult2x = xiB-xiBi;

			// Apply box filter to this pixel
			DoublePixel totSum(0,0,0);

			for (int y2 = yiTi; y2 < yiBi+1; y2++)
			{
				// Store sum of pixels for this row
				DoublePixel rowSum(0,0,0);

				rowSum += DoublePixel(i.get(xiTi,y2)) * pixMult1x;
				for (int x2 = xiTi+1; x2 < xiBi; x2++)
				{
					rowSum += DoublePixel(i.get(xiTi,y2));
				}

				// Add last pixel if within view
				if (xiBi+1 < i.w())
				{
					rowSum += DoublePixel(i.get(xiBi+1,y2)) * pixMult2x;
				}

				// Adjust if first or last row
				if (y2 == yiTi)
				{
					rowSum = rowSum * pixMult1y;
				} else if (y2 == yiBi) {
					rowSum = rowSum * pixMult2y;
				}

				totSum += rowSum;
			}

			// Normalize to get average value pixel value
			DoublePixel dpAvg = totSum*(1.0/((xiB - xiT)*(yiB - yiT)));
			set(x,y,dpAvg.toPixel());
		}
	}
}


// Resize the current image
void SimpleImage::resampleBilinear(const SimpleImage& i)
{
	// Determine multiple to convert coordinates
	// from current image to i.
	double wMult = (double)i.w() / w_;
	double hMult = (double)i.h() / h_;

	for (int y = 0; y < h_; y++)
	{
		for (int x = 0; x < w_; x++)
		{			
			set(x,y, i.getBilinear(x*wMult, y*hMult));
		}
	}
}

void SimpleImage::snapshot(HDC hdc, int xpos, int ypos, int w, int h)
{
	// Create DC/bitmap
	HDC hdcNew = CreateCompatibleDC(hdc);

	// Create DIB section
	BITMAPINFO bi;
	ZeroMemory(&bi, sizeof(BITMAPINFO));
    bi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
    bi.bmiHeader.biWidth = w;
    bi.bmiHeader.biHeight = h;
    bi.bmiHeader.biPlanes = 1;
    bi.bmiHeader.biBitCount = 32;
    bi.bmiHeader.biCompression = BI_RGB;
	bi.bmiHeader.biClrUsed = 0;
	bi.bmiHeader.biSizeImage = w_*h_*sizeof(Pixel);
	unsigned char* bits;
	HBITMAP bmp = CreateDIBSection(hdcNew, &bi, DIB_RGB_COLORS, (void**)&bits, NULL, 0);
	HGDIOBJ orig = SelectObject(hdcNew, bmp);

	// Copy image into DC
	BitBlt(hdcNew, 0, 0, w, h, hdc, xpos, ypos, SRCCOPY);

	// Copy image into this object
	delete data_;
	w_ = w;
	h_ = h;
	data_ = new Pixel[w_*h_];

	// int res = GetDIBits(hdcNew, bmp, 0, h, data_, &bi, DIB_RGB_COLORS);

	// memcpy(data_,bits,w_*h_*sizeof(Pixel));

	// Copy pixels (from BGRA to RGBA)
	for (int y = 0; y < h; y++)
	{
		for (int x = 0; x < w; x++)
		{
			data_[y*w + x].p[0] = bits[(y*w + x)*4 + 2];
			data_[y*w + x].p[1] = bits[(y*w + x)*4 + 1];
			data_[y*w + x].p[2] = bits[(y*w + x)*4 + 0];
			data_[y*w + x].p[3] = bits[(y*w + x)*4 + 3];
		}
	}


	// Release new DCs/bmps
	SelectObject(hdcNew, orig);
	DeleteObject(bmp);
	DeleteDC(hdcNew);
}


void SimpleImage::clear(Pixel p)
{
	const int imgSize = w()*h();
	for (int i = 0; i < imgSize; i++)
		data_[i] = p;
}

void SimpleImage::blit(SimpleImage* im)
{
	int w2 = min(im->w(), w());
	int h2 = min(im->h(), h());
	for (int y = 0; y < h2; y++)
	{
		memcpy(&data_[y*w()], &im->data()[y*im->w()], w2*sizeof(Pixel));
	}
}