#include "stdafx.h"

#include "ImageProvider.h"
#include "Image.h"
#include "util.h"

#include <corona.h>

SimpleImage* ImageProvider::getImage()
{
	SimpleImage* si = new SimpleImage(256, 256);
	Pixel color1 = {{rand()%256,rand()%256, rand()%256, 255}};
	Pixel color2 = {{255-color1.p[0],255-color1.p[1],255-color1.p[2], 255}};

	for (int y = 0; y < si->h(); y++)
		for (int x = 0; x < si->w(); x++)
			si->set(x,y,((x&0x10) ^ (y&0x10)) ? color1 : color2);

	return si;
}

SynchronizedObject::SynchronizedObject()
{
	hLock = CreateMutex(NULL, FALSE, NULL);
}

SynchronizedObject::~SynchronizedObject()
{
	ReleaseMutex(hLock);
}

void SynchronizedObject::lock()
{
	WaitForSingleObject(hLock, INFINITE);
}

void SynchronizedObject::unlock()
{
	ReleaseMutex(hLock);
}



RandomImageProvider::RandomImageProvider(FileList fileList)
{
	// Initialize randomizer
	Sleep(100);
	srand(timeGetTime()); rand(); rand(); rand(); rand();

	// Find files
	TCHAR pattern[MAX_PATH+10];
	TCHAR file[MAX_PATH+10];

	for (FileIt it = fileList.begin(); it != fileList.end(); ++it)
	{
		addDirectory(it->c_str());
	}

	iCurFile = 0;
	iCount = 0;

	randomizeImages();
	randomizeImages();
}

void RandomImageProvider::addDirectory(std::string dir)
{
	TCHAR pattern[MAX_PATH+10];
	TCHAR file[MAX_PATH+10];

	WIN32_FIND_DATA fd;
	_stprintf(pattern, "%s\\*", dir.c_str());
	HANDLE hSearch = FindFirstFile(pattern, &fd);
	if (hSearch == INVALID_HANDLE_VALUE) 
		return;

	while (FindNextFile(hSearch, &fd))
	{
		_stprintf(file, "%s\\%s", dir.c_str(), fd.cFileName);

		// Check if special file
		if (_tcscmp(fd.cFileName, _T(".")) == 0 ||
			_tcscmp(fd.cFileName, _T("..")) == 0)
			continue;

		// Check file type
		if (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
		{
			addDirectory(file);
		} else {
			files.push_back(file);
		}
	}

	FindClose(hSearch);
}


void RandomImageProvider::randomizeImages()
{
	for (int i = 0; i < (int)files.size()-1; i++)
	{
		int randInd = (int)randRange(i+1, files.size());
		std::swap(files[i], files[randInd]);
	}
}


SimpleImage* RandomImageProvider::getImage()
{
	// TODO: Check if no images loaded or available
	while(iCurFile != -1)
	{
		std::string fname;

		lock();

			// Randomize images if necessary
			if (iCurFile >= files.size())
			{
				// Check if valid file available
				if (iCount <= 0)
				{
					iCurFile = -1;
					unlock();
					break;
				}

				iCurFile = 0;
				iCount = 0;
				randomizeImages();
			}


			// Get filename of image
			fname = files[iCurFile++];

		unlock();



		SimpleImage* si;

		std::auto_ptr<corona::Image> i(corona::OpenImage(fname.c_str(), corona::PF_R8G8B8A8));
		if (i.get() && i->getFormat() == corona::PF_R8G8B8A8)
		{
			// Load and flip image
			si = new SimpleImage(i->getWidth(), i->getHeight());
			int h = i->getHeight();
			int w = i->getWidth();
			for (int y = 0; y < h; y++)
				memcpy(si->data() + y*w, (Pixel*)i->getPixels() + (h-y-1)*w, 4*i->getWidth());

			iCount++;
			return si;
		}
	}

	// If we are here, that means we don't have a valid image
	// Fallback to default image
	return ImageProvider::getImage();
}



