#include <stdafx.h>

#include "LoadThread.h"
#include "Image.h"
#include "ImageProvider.h"
#include "PrefetchTexture.h"
#include "Slide.h"
#include "util.h"
#include "corona.h"

#include <mmsystem.h>
#include <algorithm>
#include <vector>
#include <time.h>

// #include <gdiplus.h>

LoadThread::LoadThread(ImageProvider* ip, int mSlides, TexturePool* p, HDC dc) :
	imgProvider(ip), maxSlides(mSlides)
{
	slideLock = slideFreeSem = slideLoadSem = 0;
	thread = 0;
	threadId = 0;
	pool = p;
	hdc = dc;

	slideLock = CreateMutex(NULL, FALSE, NULL);
	slideFreeSem = CreateSemaphore(NULL, maxSlides, maxSlides, NULL);
	slideLoadSem = CreateSemaphore(NULL, 0, maxSlides, NULL);
}

LoadThread::~LoadThread()
{
	// Clean up unused slides
	typedef std::list<Slide*>::iterator TexIt;
	for (TexIt it = slides.begin(); it != slides.end(); ++it)
		delete (*it);

	// Clean up threads
	if (slideLock)		CloseHandle(slideLock);
	if (slideFreeSem)	CloseHandle(slideFreeSem);
	if (slideLoadSem)	CloseHandle(slideLoadSem);
}

void LoadThread::start()
{
	bStopThread = false;
	thread = CreateThread(NULL, 0, LoadThread::SlideThread, this, 0, &threadId);
	
	SetThreadPriority(thread,THREAD_PRIORITY_BELOW_NORMAL);
	// SetThreadPriority(thread,THREAD_PRIORITY_LOWEST);
}

void LoadThread::stop()
{
	// Stop thread
	bStopThread = true;
	WaitForSingleObject(thread, 10000); 
}

DWORD CheckerSlideThread();

DWORD WINAPI LoadThread::SlideThread(LPVOID lt)
{
	return ((LoadThread*)lt)->SlideThreadLocal();
}

DWORD LoadThread::SlideThreadLocal()
{
	// Create an opengl context so we can use glu functions
	HGLRC hglrcThread = wglCreateContext(hdc);
	wglMakeCurrent(hdc,hglrcThread);

	DWORD res = FileThread();

	wglMakeCurrent(NULL,NULL);
	wglDeleteContext(hglrcThread);

	return res;
}


DWORD LoadThread::FileThread()
{
	bool needFile = false;
	while (!bStopThread)
	{
		// Wait till spot available for image
		DWORD waitRes;
		do {
			waitRes = WaitForSingleObject(slideFreeSem, 100);
			if (bStopThread || waitRes == WAIT_FAILED)
				return 0;
		} while (waitRes == WAIT_TIMEOUT);

		SimpleImage* si = imgProvider->getImage();

		/*WaitForSingleObject(slideLock, INFINITE);
		slides.push_back(new PrefetchTexture(si, pool));
		ReleaseMutex(slideLock);
		ReleaseSemaphore(slideLoadSem, 1, NULL);*/
		addSlide(si);
	}

	return 0;
}

void LoadThread::addSlide(SimpleImage* im, bool forceCenter)
{
	WaitForSingleObject(slideLock, INFINITE);
	// slides.push_back(new PrefetchTexture(im, pool));
	slides.push_back(slideFactory.createSlide(new PrefetchTexture(im, pool), forceCenter));
	ReleaseMutex(slideLock);
	ReleaseSemaphore(slideLoadSem, 1, NULL);
}

/*
DWORD LoadThread::CheckerSlideThread()
{
	while (1)
	{
		DWORD waitRes;

		do {
			waitRes = WaitForSingleObject(slideFreeSem, 100);
			if (bStopThread)
				return 0;
		} while (waitRes == WAIT_TIMEOUT);

		SimpleImage* si = new SimpleImage(256, 256);
		Pixel color1 = {{rand()%256,rand()%256, rand()%256, 255}};
		Pixel color2 = {{255-color1.p[0],255-color1.p[1],255-color1.p[2], 255}};

		for (int y = 0; y < si->h(); y++)
			for (int x = 0; x < si->w(); x++)
				si->set(x,y,((x&0x10) ^ (y&0x10)) ? color1 : color2);

		WaitForSingleObject(slideLock, INFINITE);
		slides.push_back(new PrefetchTexture(si, pool));
		ReleaseMutex(slideLock);
		ReleaseSemaphore( slideLoadSem, 1, NULL);
	}
}*/

Slide* LoadThread::getNextSlide()
{
	WaitForSingleObject(slideLoadSem, INFINITE);
	WaitForSingleObject(slideLock, INFINITE);
	// PrefetchTexture* ti = *slides.begin();
	// Slide* s = new Slide(ti);
	Slide* s = *slides.begin();
	slides.pop_front();
	ReleaseMutex(slideLock);
	ReleaseSemaphore(slideFreeSem, 1, NULL);
	return s;
}

int LoadThread::slidesReady()
{
	return slides.size();
}
