#include <stdafx.h>

#include "config.h"
#include "Slide.h"
#include "PrefetchTexture.h"
#include "util.h"

#include <math.h>

SlideFactory::SlideFactory() : lastZoom(true), lastX(0.0), lastY(0.0)
{
}

Slide* SlideFactory::createSlide(PrefetchTexture* lt, bool forceCenter)
{
	Slide* s = new Slide(lt);
	s->randKenBurns(lastZoom, lastX, lastY, forceCenter);

	lastZoom = !lastZoom;
	lastX = s->x;
	lastY = s->y;

	return s;
}

Slide::Slide(PrefetchTexture* lt)
	: texture(lt), velX(0), velY(0), velZ(0), x(0), y(0), z(0)
{
	texture = lt;	
	timeLeft = config.secondsPerImage;
	// init();
}

Slide::~Slide()
{
	delete texture;
}

void Slide::randKenBurns(bool lastZoom, double lastX, double lastY, bool forceCenter)
{
	x = randRange(-0.15,0.15);
	y = randRange(-0.15,0.15);
	z = randRange(-0.85,-0.75);

	if (fabs(lastX) > 0.10 && x*lastX > 0)
		x = -x;
	if (fabs(lastY) > 0.10 && y*lastY > 0)
		y = -y;


	// timeLeft = 7.0;
	timeLeft = config.secondsPerImage;
	velX = -x/timeLeft;
	velY = -y/timeLeft;
	velZ = (-1-z)/timeLeft;

	if (lastZoom)
	{
		velX = -velX;
		velY = -velY;
		velZ = -velZ;
	}

	if (lastZoom || forceCenter)
	{
		x = y = 0;
		z = -1;
	}
}

void Slide::draw(double time)
{
	x += velX*time;
	y += velY*time;
	z += velZ*time;
	glPushMatrix();
	glTranslatef(x,y,z);
	texture->Draw();
	glPopMatrix();	

	timeLeft -= time;
}

