#include <stdafx.h>

#include "TexturePool.h"

#include <math.h>

TexturePool::TexturePool(
  int reserve, GLint iFormat, GLsizei w, GLsizei h, Pixel bc, bool mipmap) :
	reserveSize(reserve), internalFormat(iFormat),
	width(w), height(h), bMipmap(mipmap), poolSize(0)
{
	for (int i = 0; i < 4; i++)
		borderColor[i] = bc.p[i]/255.0;
	nLevels = bMipmap ? ((int)(log(max(width,height))/log(2))+1) : 1;
}

GLuint TexturePool::obtain()
{
	// Generate textures up to pool size or extra if neccessary
	while (poolSize < reserveSize || texidPool.empty())
	{
		// Generate texture
		GLuint tid;
		glGenTextures(1, &tid);
		glBindTexture(GL_TEXTURE_2D, tid);


		// Generate each level of texture
		int mipWidth = width;
		int mipHeight = height;
		for (int i = 0; i < nLevels; i++)
		{
			glTexImage2D(GL_TEXTURE_2D, i, internalFormat, mipWidth, mipHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);
			mipWidth = mipWidth/2;
			mipWidth = (mipWidth > 0) ? mipWidth : 1;
			mipHeight = mipHeight/2;
			mipHeight = (mipHeight > 0) ? mipHeight : 1;
		}

		// Set texture to clamp at edges
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );

		// Setup filtering parameters
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		if (bMipmap)
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
		else
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);

		// Set border color to background				
		glTexParameterfv(GL_TEXTURE_2D, GL_TEXTURE_BORDER_COLOR, borderColor);

		// Add texture to pool
		texidPool.push_back(tid);
		poolSize++;
	}

	GLuint tid = *texidPool.begin();
	texidPool.pop_front();
	return tid;
}


void TexturePool::release(GLuint tid)
{
	if (poolSize-1 < reserveSize)
	{
		// Put back in pool
		texidPool.push_back(tid);
	} else {
		// Pool full, so delete texture
		glDeleteTextures(1, &tid);
		poolSize--;
	}
}

