#ifndef TexturePool_h
#define TexturePool_h

#include <GL/gl.h>
#include <list>

#include "Image.h"

class TexturePool
{
public:
	// Create texture pool
	TexturePool(int reserve, GLint internalFormat, GLsizei w, GLsizei h, Pixel borderColor, bool mipmap);

	// Obtain a texture
	GLuint obtain();

	// Release a texture
	void release(GLuint texid);
	
	int numLevels() const { return nLevels; }

	int w() const { return width; }
	int h() const { return height; }
	bool useMipmap() const { return bMipmap; } 

private:
	int reserveSize;			// Number of elements reserved in pool
	int poolSize;				// Count of allocated textures
	int nLevels;				// Number of mipmap levels used
	GLint internalFormat;		// Internal OpenGL texture format
	GLsizei width, height;		// Width and height of the texture
	bool bMipmap;				// Specifies whether or not using mipmaps
	std::list<GLuint> texidPool;// List of textures available
	float borderColor[4];
};

#endif
