#include "stdafx.h"

#include "config.h"
#include "iSlideshow.h"
#include "../Savergl/savergl.h"
#include "resource.h"
#include "display.h"

#include <regstr.h>
#include <commctrl.h>
#include <shellapi.h>
#include <shlobj.h>
#include <stdio.h>
#include <algorithm>

LPCTSTR registryPath = ("Software\\iSlideshow");
SlideshowConfig config;


const GLsizei texDimList[] = { 256, 512, 1024, 2048 };
const int NUM_TEXDIMS = sizeof(texDimList)/sizeof(int);

const GLint texDepthList[] = { 4, GL_RGB4, GL_RGB8};
const char* texDepthListStr[] = { "Default", "16-bit", "32-bit"};
const int NUM_TEXDEPTHS = sizeof(texDepthList)/sizeof(int);


SlideshowConfig::HwndMap SlideshowConfig::hwndMap;

int openConfigBox(HWND parent){
	return config.openConfigBox(parent);
}

SlideshowConfig::SlideshowConfig()
{
	hwnd = 0;
}

SlideshowConfig::~SlideshowConfig()
{
	if (hwnd)
		hwndMap.erase(hwnd);
}

int SlideshowConfig::openConfigBox(HWND parent)
{
	INT_PTR res = DialogBoxParam(mainInstance, MAKEINTRESOURCE(DLG_SCRNSAVECONFIGURE),
		parent, (DLGPROC)ScreenSaverConfigureDialogGlobal, LPARAM(this));
	if (hwnd) {
		hwndMap.erase(hwnd);
		hwnd = 0;
	}
	return res;
}

void SlideshowConfig::setDefaults(){
	dPriority = 0;

	texDim = 2;
	texDepth = 0;
	secondsPerImage = 10;

	backRed = 0;
	backGreen = 0;
	backBlue = 0;

	multimon = false;

	/*
	FileName imageDir;
	// Get My Pictures or My Documents directory if none exists
	if (
		SHGetFolderPath(NULL, CSIDL_MYPICTURES, NULL, 
			SHGFP_TYPE_CURRENT, imageDir) != S_OK)
	{
		SHGetFolderPath(NULL, CSIDL_PERSONAL, NULL, 
			SHGFP_TYPE_CURRENT, imageDir);
	}*/
}


// Initialize all user-defined stuff
void SlideshowConfig::readRegistry(){
	LONG result;
	HKEY skey;
	DWORD valtype, valsize, val;

	setDefaults();
	theVideoMode = 10000;

	result = RegOpenKeyEx(HKEY_CURRENT_USER, registryPath, 0, KEY_READ, &skey);
	if(result != ERROR_SUCCESS)
		return;

	valsize=sizeof(val);

	result = RegQueryValueEx(skey, "Priority", 0, &valtype, (LPBYTE)&val, &valsize);
	if(result == ERROR_SUCCESS)
		dPriority = val;
	result = RegQueryValueEx(skey, "theVideoMode", 0, &valtype, (LPBYTE)&val, &valsize);
	if(result == ERROR_SUCCESS)
		theVideoMode = val;
	result = RegQueryValueEx(skey, "texDim", 0, &valtype, (LPBYTE)&val, &valsize);
	if(result == ERROR_SUCCESS)
		texDim = val;
	result = RegQueryValueEx(skey, "texDepth", 0, &valtype, (LPBYTE)&val, &valsize);
	if(result == ERROR_SUCCESS)
		texDepth = val;
	result = RegQueryValueEx(skey, "secondsPerImage", 0, &valtype, (LPBYTE)&val, &valsize);
	if(result == ERROR_SUCCESS)
		secondsPerImage = val;
	result = RegQueryValueEx(skey, "backRed", 0, &valtype, (LPBYTE)&val, &valsize);
	if(result == ERROR_SUCCESS)
		backRed = val;
	result = RegQueryValueEx(skey, "backGreen", 0, &valtype, (LPBYTE)&val, &valsize);
	if(result == ERROR_SUCCESS)
		backGreen = val;
	result = RegQueryValueEx(skey, "backBlue", 0, &valtype, (LPBYTE)&val, &valsize);
	if(result == ERROR_SUCCESS)
		backBlue = val;
	result = RegQueryValueEx(skey, "multimon", 0, &valtype, (LPBYTE)&val, &valsize);
	if(result == ERROR_SUCCESS)
		multimon = val;

	int nSize;
	result = RegQueryValueEx(skey, "DirCount", 0, &valtype, (LPBYTE)&nSize, &valsize);
	if(result == ERROR_SUCCESS)
	{
		for (int i = 0; i < nSize; i++)
		{
			TCHAR keyName[100];
			_stprintf(keyName, "Dir%03d", i+1);
			FileName fn;
			valsize = MAX_PATH;
			result = RegQueryValueEx(skey, keyName, 0, &valtype, (LPBYTE)&fn, &valsize);
			if (result == ERROR_SUCCESS)
				fileList.push_back(fn);
		}
	}

	RegCloseKey(skey);
}


// Save all user-defined stuff
void SlideshowConfig::writeRegistry(){
    LONG result;
	HKEY skey;
	DWORD val, disp;

	result = RegCreateKeyEx(HKEY_CURRENT_USER, registryPath, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE | KEY_READ, NULL, &skey, &disp);
	if(result != ERROR_SUCCESS)
		return;

	val = dPriority;
	RegSetValueEx(skey, "Priority", 0, REG_DWORD, (CONST BYTE*)&val, sizeof(val));
	val = theVideoMode;
	RegSetValueEx(skey, "theVideoMode", 0, REG_DWORD, (CONST BYTE*)&val, sizeof(val));
	val = texDim;
	RegSetValueEx(skey, "texDim", 0, REG_DWORD, (CONST BYTE*)&val, sizeof(val));
	val = texDepth;
	RegSetValueEx(skey, "texDepth", 0, REG_DWORD, (CONST BYTE*)&val, sizeof(val));
	val = secondsPerImage;
	RegSetValueEx(skey, "secondsPerImage", 0, REG_DWORD, (CONST BYTE*)&val, sizeof(val));
	val = backRed;
	RegSetValueEx(skey, "backRed", 0, REG_DWORD, (CONST BYTE*)&val, sizeof(val));
	val = backGreen;
	RegSetValueEx(skey, "backGreen", 0, REG_DWORD, (CONST BYTE*)&val, sizeof(val));
	val = backBlue;
	RegSetValueEx(skey, "backBlue", 0, REG_DWORD, (CONST BYTE*)&val, sizeof(val));
	val = multimon;
	RegSetValueEx(skey, "multimon", 0, REG_DWORD, (CONST BYTE*)&val, sizeof(val));

	// Remove previous entries
	DWORD valsize, valtype;
	valsize = sizeof(DWORD);
	int nSize;
	result = RegQueryValueEx(skey, "DirCount", 0, &valtype, (LPBYTE)&nSize, &valsize);
	if(result == ERROR_SUCCESS)
	{
		for (int i = 0; i < nSize; i++)
		{
			TCHAR keyName[100];
			_stprintf(keyName, "Dir%03d", i+1);
			RegDeleteValue(skey, keyName);
		}
	}

	// Add new directories
	val = fileList.size();
	RegSetValueEx(skey, "DirCount", 0, REG_DWORD, (CONST BYTE*)&val, sizeof(val));
	int fileNum = 1;
	for (FileIt it = fileList.begin(); it != fileList.end(); ++it)
	{
		TCHAR keyName[100];
		_stprintf(keyName, "Dir%03d", fileNum);
		const TCHAR *val = it->c_str();
		RegSetValueEx(skey, keyName, 0, REG_SZ, (CONST BYTE*)val, _tcslen(it->c_str())*sizeof(TCHAR));
		fileNum++;
	}

	RegCloseKey(skey);
}

BOOL CALLBACK SlideshowConfig::aboutProc(HWND hdlg, UINT msg, WPARAM wpm, LPARAM lpm){
	switch(msg){
	case WM_CTLCOLORSTATIC:
		if(HWND(lpm) == GetDlgItem(hdlg, WEBPAGE)){
			SetTextColor(HDC(wpm), RGB(0,0,255));
			SetBkColor(HDC(wpm), COLORREF(GetSysColor(COLOR_3DFACE)));
			return(int(GetSysColorBrush(COLOR_3DFACE)));
		}
		break;
    case WM_COMMAND:
		switch(LOWORD(wpm)){
		case IDOK:
		case IDCANCEL:
			EndDialog(hdlg, LOWORD(wpm));
			break;
		/*case WEBPAGE:
			ShellExecute(NULL, "open", "http://business.fortunecity.com/knight/352/", NULL, NULL, SW_SHOWNORMAL);*/
		}
	}
	return FALSE;
}


void SlideshowConfig::initControls(HWND hdlg){
	char cval[100];

	// Setup priority range
	SendDlgItemMessage(hdlg, PRIORITY, TBM_SETRANGE, 0, LPARAM(MAKELONG(DWORD(0), DWORD(2))));
	SendDlgItemMessage(hdlg, PRIORITY, TBM_SETPOS, 1, LPARAM(dPriority));
	SendDlgItemMessage(hdlg, PRIORITY, TBM_SETLINESIZE, 0, LPARAM(1));
	SendDlgItemMessage(hdlg, PRIORITY, TBM_SETPAGESIZE, 0, LPARAM(1));
	sprintf(cval, "%d", dPriority);
	SendDlgItemMessage(hdlg, PRIORITYTEXT, WM_SETTEXT, 0, LPARAM(cval));


	// Setup frame time range
	SendDlgItemMessage(hdlg, IDC_FRAMETIME_SLIDER, TBM_SETRANGE, 0, LPARAM(MAKELONG(DWORD(5), DWORD(20))));
	SendDlgItemMessage(hdlg, IDC_FRAMETIME_SLIDER, TBM_SETPOS, 1, LPARAM(secondsPerImage));
	SendDlgItemMessage(hdlg, IDC_FRAMETIME_SLIDER, TBM_SETLINESIZE, 0, LPARAM(1));
	SendDlgItemMessage(hdlg, IDC_FRAMETIME_SLIDER, TBM_SETPAGESIZE, 0, LPARAM(1));
	sprintf(cval, "%d", secondsPerImage);
	SendDlgItemMessage(hdlg, IDC_FRAMETIME_SLIDER_TEXT, WM_SETTEXT, 0, LPARAM(cval));

	// Setup color range
	SendDlgItemMessage(hdlg, IDC_REDBACK_SLIDER, TBM_SETRANGE, 0, LPARAM(MAKELONG(DWORD(0), DWORD(255))));
	SendDlgItemMessage(hdlg, IDC_REDBACK_SLIDER, TBM_SETPOS, 1, LPARAM(backRed));
	SendDlgItemMessage(hdlg, IDC_GREENBACK_SLIDER, TBM_SETRANGE, 0, LPARAM(MAKELONG(DWORD(0), DWORD(255))));
	SendDlgItemMessage(hdlg, IDC_GREENBACK_SLIDER, TBM_SETPOS, 1, LPARAM(backGreen));
	SendDlgItemMessage(hdlg, IDC_BLUEBACK_SLIDER, TBM_SETRANGE, 0, LPARAM(MAKELONG(DWORD(0), DWORD(255))));
	SendDlgItemMessage(hdlg, IDC_BLUEBACK_SLIDER, TBM_SETPOS, 1, LPARAM(backBlue));

	// Set multimon checkbox
	CheckDlgButton(hdlg, IDC_MULTIMON, multimon ? BST_CHECKED : BST_UNCHECKED);

	// Setup texture dimension combo boxes
	SendDlgItemMessage(hdlg, IDC_TEXDIM_COMBO, CB_RESETCONTENT, 0, 0);
	for (int i = 0; i < NUM_TEXDIMS; i++)
	{
		// Convert to string
		sprintf(cval, "%dx%d", texDimList[i], texDimList[i]);
		SendDlgItemMessage(hdlg, IDC_TEXDIM_COMBO, CB_ADDSTRING, 0, LPARAM(cval));
	}
	SendDlgItemMessage(hdlg, IDC_TEXDIM_COMBO, CB_SETCURSEL, WPARAM(texDim), 0);


	// Setup texture depth combo boxes
	SendDlgItemMessage(hdlg, IDC_TEXDEPTH_COMBO, CB_RESETCONTENT, 0, 0);
	for (int i = 0; i < NUM_TEXDEPTHS; i++)
	{
		SendDlgItemMessage(hdlg, IDC_TEXDEPTH_COMBO, CB_ADDSTRING, 0, LPARAM(texDepthListStr[i]));
	}
	SendDlgItemMessage(hdlg, IDC_TEXDEPTH_COMBO, CB_SETCURSEL, WPARAM(texDepth), 0);

	// Setup image directories
	SendDlgItemMessage(hdlg, IDC_IMAGE_DIRS, LB_RESETCONTENT, 0, 0);
	for (FileIt it = fileList.begin(); it != fileList.end(); it++)
	{
		SendDlgItemMessage(hdlg, IDC_IMAGE_DIRS, LB_ADDSTRING, 0, LPARAM(it->c_str()));
	}

	InitVideoModeComboBox(hdlg, VIDEOMODE);
}

void SlideshowConfig::removeDirectories(HWND hdlg)
{
	int nSel = SendDlgItemMessage(hdlg, IDC_IMAGE_DIRS, LB_GETSELCOUNT, 0, 0);
	int* sel = new int[nSel];
	SendDlgItemMessage(hdlg, IDC_IMAGE_DIRS, LB_GETSELITEMS, nSel, (LPARAM)sel);
	std::sort(sel, sel+nSel);

	for (int i = nSel-1; i >= 0; i--)
	{
		SendDlgItemMessage(hdlg, IDC_IMAGE_DIRS, LB_DELETESTRING, sel[i], 0);
	}
	delete [] sel;
}

BOOL SlideshowConfig::ScreenSaverConfigureDialogGlobal(HWND hdlg, UINT msg, WPARAM wpm, LPARAM lpm){
	// Create object if new dialog
	if (msg == WM_INITDIALOG)
	{
		SlideshowConfig* thisPtr = (SlideshowConfig*)lpm;
		thisPtr->hwnd = hdlg;
		hwndMap.insert(std::make_pair(hdlg, thisPtr));
	}

	// Lookup window
	HwndMapIterator it = hwndMap.find(hdlg);
	if (it == hwndMap.end())
		return FALSE;
	else
		return it->second->ScreenSaverConfigureDialog(hdlg, msg, wpm, lpm);
}

BOOL SlideshowConfig::ScreenSaverConfigureDialog(HWND hdlg, UINT msg, WPARAM wpm, LPARAM lpm){
	int ival;
	char cval[16];

    switch(msg){
    case WM_INITDIALOG:
        InitCommonControls();
        readRegistry();
		initControls(hdlg);
        return TRUE;

	case WM_DROPFILES:
	{
		// Get dropped files
		HDROP hDrop = (HDROP)wpm;
		int nFiles = DragQueryFile(hDrop, 0xFFFFFFFF, NULL, 0);
		for (int i = 0; i < nFiles; i++)
		{
			FileName fn;
			DragQueryFile(hDrop, i, fn, MAX_PATH);

			// Check if a directory
			if (GetFileAttributes(fn) & FILE_ATTRIBUTE_DIRECTORY)
				SendDlgItemMessage(hdlg, IDC_IMAGE_DIRS, LB_ADDSTRING, 0, (LPARAM)fn);
		}
		DragFinish(hDrop);
		return TRUE;
	}

	case WM_VKEYTOITEM:
	{
		if ((HWND)lpm == GetDlgItem(hdlg, IDC_IMAGE_DIRS))
		{
			switch(LOWORD(wpm))
			{
			case VK_DELETE:
				removeDirectories(hdlg);
				return TRUE;
			}
			return -1;
		}
		break;
	}

	case WM_CHARTOITEM:
		return -1;

    case WM_COMMAND:
        switch(LOWORD(wpm)){
        case IDOK:
		{
			// SendDlgItemMessage(hdlg, IDC_IMAGE_DIR, WM_GETTEXT, WPARAM(MAX_PATH+10), LPARAM(imageDir));
			dPriority = SendDlgItemMessage(hdlg, PRIORITY, TBM_GETPOS, 0, 0);
			theVideoMode = SendDlgItemMessage(hdlg, VIDEOMODE, CB_GETCURSEL, 0, 0);
			secondsPerImage = SendDlgItemMessage(hdlg, IDC_FRAMETIME_SLIDER, TBM_GETPOS, 0, 0);
			texDim = SendDlgItemMessage(hdlg, IDC_TEXDIM_COMBO, CB_GETCURSEL, 0, 0);
			texDepth = SendDlgItemMessage(hdlg, IDC_TEXDEPTH_COMBO, CB_GETCURSEL, 0, 0);

			backRed = SendDlgItemMessage(hdlg, IDC_REDBACK_SLIDER, TBM_GETPOS, 0, 0);
			backGreen = SendDlgItemMessage(hdlg, IDC_GREENBACK_SLIDER, TBM_GETPOS, 0, 0);
			backBlue = SendDlgItemMessage(hdlg, IDC_BLUEBACK_SLIDER, TBM_GETPOS, 0, 0);

			multimon = (IsDlgButtonChecked(hdlg, IDC_MULTIMON) == BST_CHECKED);

			fileList.clear();
			int nFiles = SendDlgItemMessage(hdlg, IDC_IMAGE_DIRS, LB_GETCOUNT, 0, 0);
			for (int i = 0; i < nFiles; i++)
			{
				FileName fn;
				SendDlgItemMessage(hdlg, IDC_IMAGE_DIRS, LB_GETTEXT, i, LPARAM(fn));
				fileList.push_back(fn);
			}

			writeRegistry();
            // Fall through
		}
        case IDCANCEL:
            EndDialog(hdlg, LOWORD(wpm));
            break;
		case DEFAULTS:
			setDefaults();
			initControls(hdlg);
			break;
		case IDC_REMOVE_DIR:
		{
			removeDirectories(hdlg);
			break;
		}

		case IDC_ADD_DIR:
		{
			// Prompt user for folder
			FileName fn;
			BROWSEINFO bi;
			bi.hwndOwner = hdlg;
			bi.pidlRoot = NULL;
			bi.lpszTitle = "Select image directory";
			bi.ulFlags = 0;
			bi.lpfn = NULL;
			bi.lParam = 0;
			bi.pszDisplayName = fn;
			bi.iImage = 0;
			LPITEMIDLIST id = SHBrowseForFolder(&bi);

			if (!id)
				return TRUE;

			// SendDlgItemMessage(hdlg, IDC_IMAGE_DIRS, LB_ADDSTRING, 0, (LPARAM)fn);

			// Get folder name
			FileName fnFull;
			if ( SHGetPathFromIDList (id, fnFull) )
			{
				SendDlgItemMessage(hdlg, IDC_IMAGE_DIRS, LB_ADDSTRING, 0, (LPARAM)fnFull);
			}

			// Free data
			LPMALLOC pMalloc = 0;
			if (SHGetMalloc(&pMalloc) == NOERROR)
			{
				pMalloc->Free(id);
				pMalloc->Release();
			}

			return TRUE;
		}


        case ABOUT:
			DialogBox(mainInstance, MAKEINTRESOURCE(DLG_ABOUT), hdlg, DLGPROC(aboutProc));
		}
        return TRUE;
	case WM_HSCROLL:
		if(HWND(lpm) == GetDlgItem(hdlg, WIDTH)){
			ival = SendDlgItemMessage(hdlg, WIDTH, TBM_GETPOS, 0, 0);
			sprintf(cval, "%d", ival);
			SendDlgItemMessage(hdlg, WIDTHTEXT, WM_SETTEXT, 0, LPARAM(cval));
		}
		if(HWND(lpm) == GetDlgItem(hdlg, SPEED)){
			ival = SendDlgItemMessage(hdlg, SPEED, TBM_GETPOS, 0, 0);
			sprintf(cval, "%d", ival);
			SendDlgItemMessage(hdlg, SPEEDTEXT, WM_SETTEXT, 0, LPARAM(cval));
		}
		if(HWND(lpm) == GetDlgItem(hdlg, PRIORITY)){
			ival = SendDlgItemMessage(hdlg, PRIORITY, TBM_GETPOS, 0, 0);
			sprintf(cval, "%d", ival);
			SendDlgItemMessage(hdlg, PRIORITYTEXT, WM_SETTEXT, 0, LPARAM(cval));
		}
		if(HWND(lpm) == GetDlgItem(hdlg, IDC_FRAMETIME_SLIDER)){
			ival = SendDlgItemMessage(hdlg, IDC_FRAMETIME_SLIDER, TBM_GETPOS, 0, 0);
			sprintf(cval, "%d", ival);
			SendDlgItemMessage(hdlg, IDC_FRAMETIME_SLIDER_TEXT, WM_SETTEXT, 0, LPARAM(cval));
		}
		return TRUE;
    }
    return FALSE;
}


GLsizei SlideshowConfig::textureDimensions()
{
	return texDimList[texDim];
}

GLint SlideshowConfig::textureDepth()
{
	return texDepthList[texDepth];
}
