#include <stdafx.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glprocs.h>
#include <mmsystem.h>
#include <shellapi.h>
#include <process.h>
#include <time.h>

#include "../Savergl/savergl.h"
#include "iSlideShow.h"
#include "display.h"
#include "Image.h"
#include "ImageProvider.h"
#include "Slide.h"
#include "util.h"

// #include <ximage.h>

#include <algorithm>
#include <list>
#include <vector>

GLint glMaxTexDim;

static void initGL()
{
	// glClearColor (0.0, 0.0, 0.0, 0.0);
	glClearColor (config.backRed/255.0, config.backGreen/255.0, config.backBlue/255.0, 0.0);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
	glFrustum(-0.25, 0.25, -0.25, 0.25, 0.25, 2.0);
	wglSwapIntervalEXT(1);
}


Slideshow::Slideshow(HWND wnd, SimpleImage* ss)
{
	mainSlide = 0;
	newSlide = 0;
	curTime = 0;
	// startTime = 0;
	hwnd = wnd;
	initSaver(ss);
}

Slideshow::~Slideshow()
{
	delete mainSlide;
	delete newSlide;

	loader->stop();
	loader.reset();
	pool.reset();

	// Kill device context
	ReleaseDC(hwnd, hdc);
	wglMakeCurrent(NULL, NULL);
	wglDeleteContext(hglrc);
}

void Slideshow::initSaver(SimpleImage* screenShot)
{
	int i = GL_TEXTURE_RECTANGLE_ARB;
	static std::auto_ptr<ImageProvider> imgProvider;

	RECT rect;

	srand((unsigned)time(NULL));
	rand(); rand(); rand(); rand(); rand();

	// Window initialization
	hdc = GetDC(hwnd);
	SetBestPixelFormat(hdc);

	// Get refresh rate
	DEVMODE mode;
	mode.dmSize = sizeof(DEVMODE);
	mode.dmDriverExtra = 0;
	EnumDisplaySettings(NULL, ENUM_CURRENT_SETTINGS, &mode);
	frameTime = 1.0 / mode.dmDisplayFrequency;

	// Setup opengl
	hglrc = wglCreateContext(hdc);
	GetClientRect(hwnd, &rect);
	wglMakeCurrent(hdc, hglrc);
	glViewport(rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top);
	initGL();

	// Setup imae provider
	if (!imgProvider.get())
	{
		imgProvider.reset(new RandomImageProvider(config.fileList));
	}

	// Setup file loader
	Pixel backColor = {config.backRed, config.backGreen, config.backBlue, 0};

	// textureTarget = GL_ARB_texture_rectangle;
	// pool.reset(new TexturePool(2, config.textureDepth(), rect.right - rect.left, rect.bottom - rect.top, backColor, false));
	pool.reset(new TexturePool(2, config.textureDepth(), config.textureDimensions(), config.textureDimensions(), backColor, false));
	loader.reset(new LoadThread(imgProvider.get(), 1, pool.get(), hdc));

	// Add screenshot of desktop if available
	if (screenShot)
		loader->addSlide(screenShot, true);

	// Setup whether to zoom this slideshow or not
	static bool lastZoom = false;
	lastZoom = !lastZoom;
	loader->getSlideFactory().lastZoom = lastZoom;

	curTime = timeGetTime()/1000.0;

	loader->start();
}


extern HWND hwndPrimary;

void Slideshow::display()
{
	wglMakeCurrent(hdc, hglrc);

	// Attempt to load first slides, or skip this frame if not ready
	double difTime;
	if (!mainSlide)
	{
		mainSlide = loader->getNextSlide();
		// newSlide = loader->getNextSlide();
		difTime = 0;
		curTime = timeGetTime()/1000.0 + 1.0;
	} else {
		double newTime = timeGetTime()/1000.0; 
		difTime = newTime-curTime;
		curTime = newTime;
		difTime = min(difTime, 0.1);
		difTime = max(difTime, 0.0);
	}

	double fadeStart = 2.0;

	// Frame limit in case opengl isn't waiting for vertical refresh
	/* if (difTime < frameTime)
	{
		Sleep(frameTime*1000);
	}*/
	

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glClear(GL_COLOR_BUFFER_BIT);

	double timeLeft = mainSlide->getTimeLeft();
	if (timeLeft < fadeStart)
	{
		if (!newSlide)
			newSlide = loader->getNextSlide();

		float fade = (fadeStart-timeLeft)/fadeStart;
		fade = min(fade, 1.0);
		glEnable (GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glColor4f(1.0,1.0,1.0, 1.0 - fade);
		mainSlide->draw(difTime);

		glColor4f(1.0,1.0,1.0, fade);
		newSlide->draw(difTime);
		glDisable (GL_BLEND);
		

		// Cross fade between slides
/*		glEnable (GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glColor4f(1.0,1.0,1.0, (fadeStart-timeLeft)/fadeStart);
		newSlide->draw(difTime);
		glDisable (GL_BLEND);*/
	} else {
		/* glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);
		glEnable (GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); */
		mainSlide->draw(difTime);
		// glDisable (GL_BLEND);
	}

	if (timeLeft < 0)
	{
		delete mainSlide;
		mainSlide = newSlide;
		newSlide = loader->getNextSlide();
	}

    // wglSwapLayerBuffers(hdc, WGL_SWAP_MAIN_PLANE);
	SwapBuffers(hdc);
}



